\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{exam-sd}[2023/4/11 Enhanced Exam class]
\LoadClass[11pt]{exam}
\newif\ifTF
\newif\ifnotes
\newif\ifpractice
\newif\ifprinting
\newif\ifthmsheet
\DeclareOption{printing}{\printingtrue}
\DeclareOption{TF}{\TFtrue}
\DeclareOption{notes}{\notestrue}
\DeclareOption{practice}{\practicetrue}
\DeclareOption{thmsheet}{\thmsheettrue}
\ProcessOptions\relax
\RequirePackage{hyperref,amsmath,amsfonts,amssymb,mathabx,mdframed,lipsum,enumerate,xparse,inputenc,paralist}
\RequirePackage[center]{titlesec}
\providecommand{\notes}{0}
\providecommand{\TF}{0}
\providecommand{\practice}{0}
\providecommand{\printing}{0}
\newcommand{\Z}{\mathbb{Z}}
\newcommand{\C}{\mathbb{C}}
\newcommand{\R}{\mathbb{R}}
\newcommand{\Q}{\mathbb{Q}}
\checkboxchar{{\Large$\Box$}}
\checkedchar{{\Large$\blacksquare$}}
\newcommand{\as}[1]{\langle #1 \rangle}
\newcommand{\qed}{\hfill{\huge$\square$}\\}
\newcommand{\smat}[1]{\left(\begin{smallmatrix} #1 \end{smallmatrix} \right)}
\newcommand{\mat}[1]{\left(\begin{matrix} #1 \end{matrix} \right)}
\newcommand\thms[2]{\noindent\textbf{#1. }\textit{#2\\}}
\newcommand{\im}{\operatorname{Im}}
\newcommand{\GL}{\operatorname{GL}}
\renewcommand{\solutiontitle}{\noindent\textbf{\underline{SOLUTION}}\par\noindent}	
\newcommand{\myleft}{\makebox[.5\textwidth]{Name:\enspace\hrulefill}}
\newcommand{\myright}{\makebox[.3\textwidth]{ID:\enspace\hrulefill}}
\newcommand{\mycopy}{}
\header{\ifprintanswers
\else
\ifthenelse{\equal{\thepage}{-1}}{\myleft}{}
\fi
}
       {}
       {\ifprintanswers
\else
\ifthenelse{\equal{\thepage}{-1}}{\myright}{}
\fi}

\firstpagefootrule
\runningfootrule
\footer{\mycopy}{}{\ifthenelse{\equal{\thepage}{-1}}{}{Page \thepage/\numpages}} 
\shadedsolutions
\definecolor{SolutionColor}{rgb}{.8,.9,1}

\def\time#1{\gdef\@time{#1}}
\def\term#1{\gdef\@term{#1}}
\renewcommand{\maketitle}{%
	\begin{center}
	\Large
	\@title \ifpractice{ Practice Problem\ifprintanswers{\;Solutions\\ \normalsize by \@author}\else{s\\\@date\\}\fi }\else{ 			
	\ifprintanswers{(\@term) Solutions\\ \normalsize by \@author}
	\else{
		\\\@date\\
		}
	\fi
	}\fi
	\end{center}
	\ifprintanswers{\setcounter{page}{1}}
	\else{\ifpractice{}
		\else{%
		\setcounter{page}{-1}
		Instructions:
		\begin{itemize}
		\item Do not start until instructed to do so.
		\item You have exactly \@time\;to complete the exam (unless you have approved accommodations). After the allotted time limit, please stop all work.
		\item Please refer to which results from the text or proven in class you are using. Exact theorem numbers are not necessary; it is enough to give the name of the result or a short description of the result. \ifthmsheet{You may use any results proven in the text or in class, including results that are not on the provided list of theorems.}\else{}\fi
		\item \ifTF{For True/False questions, please clearly mark the box for your chosen answer. If you think your mark might be ambiguous, you may write ``True'' or ``False'' next to the question to indicate your chosen answer. \item 
		For True/False questions, showing work is not required. Otherwise, please show work for partial credit. For multi-part questions, you can use part (a) to answer part (b), etc. }\else{For multi-part questions, you can use part (a) to answer part (b), etc. Please show your work for partial credit. }\fi
		\item Use of calculators, cell-phones, computers, tablets, and other electronic devices is strictly prohibited.
		\item \ifnotes{ One sheet of notes (8.5 x 11 in, single-sided, one page) is allowed, to be handed in with your exam.}\else{ No outside resources such as books or notes are allowed.\ifthmsheet{ A list of some useful theorems has been provided with your exam.}\else{}\fi}\fi
		\end{itemize} \newpage\thispagestyle{empty}\ifthmsheet{}\else{\mbox{}\newpage}\fi
}\fi

	}\fi
	
}

%
\newenvironment{ThmSheet}{
\begin{center}\Large Theorem Sheet \end{center}
}
{\newpage
}



%% The following code is based on an answer by Michael S Taylor. I have modified it somewhat.
%% https://tex.stackexchange.com/a/273839   

\newcommand{\vstretch}[1]{\vspace*{\stretch{#1}}}





\newcommand*\AnswerBox[1]{%
    \parbox[t][1cm]{0.95\textwidth}{%
    \begin{solution}#1\end{solution}}
    \vspace*{\stretch{1}}}

%inserts a page break ONLY in the version to be printed out by me; pdf version will have page breaks everywhere
\newcommand{\optbreak}{\ifprinting{\newpage}\else{}\fi}
%inserts a page break only when printing answers
\newcommand{\optbreaka}{\ifprintanswers{\newpage}\else{}\fi}

%inserts a description of common errors
\newcommand{\errors}{{\it\underline{COMMON ERRORS:}} }

%first argument is optional and makes the vspace stretch differently for each thing on the page
\NewDocumentEnvironment{AnswerPage}{O{1}}
    {\begin{minipage}[t][1cm]{\textwidth}%
    \begin{solution}}
    {\end{solution}\end{minipage}
    \vspace*{\stretch{#1}}
    }
   
%environment for a Part, default points value is 10, statement gets passed as argument    
\NewDocumentEnvironment{Part}{O{10} m}
{\part[#1] #2 \\ \begin{AnswerPage}[#1] }
{\end{AnswerPage}}

%to be used for a question that has nothing in it.
\newcommand{\firstpart}[2][10]{\part[#1] #2 \\ \begin{AnswerPage}[#1] }

%environment for a Question, star means no parts, ! means this question lacks a statement of its own and the statement/points value given in #4/#3 is actually for the first part, default points value is 20, statement gets passed as fourth argument, optional proof of first part appears as fifth argument
\NewDocumentEnvironment{Question}{s t! O{20} m o}
{\IfBooleanTF{#1}
	{  \question[#3] #4 \\\begin{AnswerPage}[#3]}
	{\IfBooleanTF{#2}
		{\question \begin{parts} \firstpart[#3]{#4} #5 \end{AnswerPage}}
		{\question #4 \begin{parts}}
	}
}
{\IfBooleanTF{#1}{\end{AnswerPage}}{\end{parts}}
	\ifprintanswers{\ifprinting{} %optbreak included for changing the printed version vs the pdf version
				\else{\newpage
				}\fi
	}
	\else{\ifpractice{}
		\else{ \newpage
			{\bf Extra space for Problem \thequestion. Please label clearly which part(s) you are solving.}
			\newpage
		}\fi
	}\fi
}










%inserts a page for general extra space
\newcommand{\extraspacepage}{\newpage
			{\bf Extra space. Please label clearly which problem(s) and part(s) you are solving.}
			\newpage}







%% The following code is based on an answer by Gonzalo Medina
%% https://tex.stackexchange.com/a/13106/39194
\CorrectChoiceEmphasis{}
\newcommand*{\TrueFalse}[1]{%
\ifprintanswers
    \ifthenelse{\equal{#1}{T}}{%
      \begin{oneparcheckboxes}  \correctchoice \textbf{TRUE} \choice False \end{oneparcheckboxes}
    }{
      \begin{oneparcheckboxes}  \choice True \correctchoice  \textbf{FALSE} \end{oneparcheckboxes}
    }
\else
    \begin{oneparcheckboxes} \choice True \choice False \end{oneparcheckboxes}
\fi
} 
\newlength\TFheight
\setlength\TFheight{20pt}
\newlength\TFlengthA
\newlength\TFlengthB
\settowidth\TFlengthA{\hspace*{.9in}}
\newcommand\TFQuestion[2]{%
    \setlength\TFlengthB{\linewidth}
    \addtolength\TFlengthB{-\TFlengthA}
    \parbox[t][\TFheight]{\TFlengthA}{\TrueFalse{#1}}\parbox[t]{\TFlengthB}{#2}}



%first argument is "true" or "false", second is the question, third is the solution
\newcommand{\tfpart}[3]{\part\TFQuestion{#1}{#2}\\
\vspace{-11pt}\\
\AnswerBox{#3}}

%optional argument for number of points. makes a question that has parts that are true-false questions and takes up a whole page. 
\newenvironment{truefalses}[1][2]
{
\renewcommand{\partshook}{\leftmargin=0pt%
  \labelwidth=-\labelsep}
\question \fullwidth{%
Fill in the box next to True or False.  You do not need to justify your answers. (#1 points each)
}\begin{parts}
}
{\end{parts}\newpage\renewcommand{\partshook}{\setlength{\leftmargin}{\leftmargini}%
  \setlength{\labelwidth}{\dimexpr \leftmargin-\labelsep}}
}    
    

%default number of copies is 50. https://tex.stackexchange.com/questions/272047/numbered-copies-with-exam-class inspired by and modified from this link
\NewDocumentEnvironment{copies}{O{50}+b}
{\renewcommand{\mycopy}{{Copy \ifnum\value{copynum}<100 0\fi\ifnum\value{copynum}<10 0\fi\arabic{copynum}}}
\newcounter{copynum} % copy number, to be printed in the footer 
\newcounter{NumCopy} 
\ifprintanswers{
	\setcounter{NumCopy}{1}
}
\else
	\ifprinting{
		\setcounter{NumCopy}{#1} 
	}
	\else
		\setcounter{NumCopy}{1}
	\fi
\fi 
\whiledo{\thecopynum<\theNumCopy}{%
  \setcounter{page}{-1} % start numbering pages for the current copy at 1
  \addtocounter{copynum}{1} % the number of the current copy
  \setcounter{question}{1} % make questions start at 1 again
  \setcounter{part}{1}  % make parts start at 1 again
  #2
  }
}
{
}
    

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    